@@ -5,12 +5,16 @@ import requests
 from ai_transform.logger import format_logging_info
 from requests.models import Response
 
-from typing import Union, Sequence, Mapping, Any
+from typing import Union, Sequence, Mapping, Callable, Any
 
 logger = logging.getLogger(__file__)
 logging.basicConfig()
 
 
+class ManualRetry(Exception):
+    pass
+
+
 def request_wrapper(
     fn: Union[requests.get, requests.post],
     args: Sequence = None,
@@ -19,6 +23,7 @@ def request_wrapper(
     timeout: int = 30,
     output_to_stdout: bool = False,  # support output to stdout to ensure logging is working
     exponential_backoff: float = 1,
+    retry_func: Callable = None,
 ) -> Response:
 
     if args is None:
@@ -27,9 +32,13 @@ def request_wrapper(
     if kwargs is None:
         kwargs = {}
 
+    if retry_func is None:
+        retry_func = lambda result: False
+
     for n in range(1, num_retries + 1):
         try:
             result = fn(*args, **kwargs)
+
             if result.status_code != 200:
                 to_log = format_logging_info(
                     {
@@ -41,9 +50,21 @@ def request_wrapper(
                     print(to_log)
                 else:
                     logger.debug(to_log)
-        except Exception as e:
+
+                raise ValueError(to_log)
+
+            if retry_func(result):
+                to_log_for_retry = "Manual Retry Triggered..."
+                if output_to_stdout:
+                    print(to_log_for_retry)
+                else:
+                    logger.debug(to_log_for_retry)
+                raise ManualRetry
+
+        except (Exception, ManualRetry) as e:
             logger.exception(e)
             time.sleep(timeout * exponential_backoff**n)
         else:
             return result
+
     return result
